#include <avr/io.h>
#include <util/delay.h>
#include "tlc5916.h"
#include "usart.h"
enum mode {NORMAL, SPECIAL};
void tlc5916_init(void) {
  LE_DDR |= (1<<LE);
  OE_DDR |= (1<<OE);
  OE_PORT |= (1<<OE);
  LE_PORT &= ~(1<<LE);
}
#define SCK_PULSE {      \
  XCK_PORT &= ~(1<<XCK); \
  XCK_PORT |= (1<<XCK);  \
  XCK_PORT &= ~(1<<XCK); \
}
void tlc5916_mode_switch(uint8_t newmode) {
  static uint8_t mode = NORMAL;
  if (newmode == SPECIAL) {
    usart_disable();
    // initial condition
    OE_PORT |= (1<<OE);
    LE_PORT &= ~(1<<LE);
    
    SCK_PULSE            // 1.
    OE_PORT &= ~(1<<OE);
    SCK_PULSE            // 2.
    OE_PORT |= (1<<OE);
    SCK_PULSE            // 3.
    LE_PORT |= (1<<LE);
    SCK_PULSE            // 4.
    LE_PORT &= ~(1<<LE);
    SCK_PULSE            // 5.
    mode = newmode;
    usart_enable();
  } else {
    usart_disable();
    // initial condition
    OE_PORT |= (1<<OE);
    LE_PORT &= ~(1<<LE);
    
    SCK_PULSE            // 1.
    OE_PORT &= ~(1<<OE);
    SCK_PULSE            // 2.
    OE_PORT |= (1<<OE);
    SCK_PULSE            // 3.
    LE_PORT &= ~(1<<LE);
    SCK_PULSE            // 4.
    LE_PORT &= ~(1<<LE);
    SCK_PULSE            // 5.
    OE_PORT &= ~(1<<OE);
    mode = newmode;
    usart_enable();
  }
}
uint8_t tlc5916_current_adjust (uint8_t step) {
  tlc5916_mode_switch (SPECIAL);
  usart_disable();
  XCK_PORT &= ~(1<<XCK);
  OE_PORT |= (1<<OE);
  LE_PORT &= ~(1<<LE);
  TXD0_DDR |= (1<<TXD0);
  uint8_t i;
  for (i = 0; i < 8; i++) {
    if (step & (1<<(7 - i)))
      TXD0_PORT |= (1<<TXD0);
    else
      TXD0_PORT &= ~(1<<TXD0);
    if (i == 7)
      LE_PORT |= (1<<LE);
    SCK_PULSE
    LE_PORT &= ~(1<<LE);
  }
  for (i = 0; i < 8; i++) {
    if (step & (1<<(7 - i)))
      TXD0_PORT |= (1<<TXD0);
    else
      TXD0_PORT &= ~(1<<TXD0);
    if (i == 7)
      LE_PORT |= (1<<LE);
    SCK_PULSE
    LE_PORT &= ~(1<<LE);
  }
  usart_enable();
  tlc5916_mode_switch (NORMAL);
  return step;
}
void tlc5916_out(uint8_t data, uint8_t device) {
  static uint8_t tlc5916_data[NUMBER_OF_TLC5916];
  uint8_t i;
  tlc5916_data[device] = data;
  for (i = 0; i < NUMBER_OF_TLC5916; i++)
    usart_mspim_transceive (tlc5916_data[i]);
  LE_PORT |= (1<<LE);
  LE_PORT &= ~(1<<LE);
  OE_PORT &= ~(1<<OE);
}
